/*
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2014

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <command/command.h>
#include <dt_port_numbers.h>

static char *host = "localhost";
static int port = CONSOLE_COMMAND_PORT;

static void handler_list_entry(struct command *command, 
    char *name, enum command_value_type type, char *unit,
    int typec, enum command_ptype typev[])
{
	int i;
	
	printf("Command name='%s', type=%s, unit='%s'\n", 
	    name, enum_command_value_type2str(type), unit);
	for (i = 0; i < typec; i++) {
		printf("\tAccepts: %s\n", enum_command_ptype2str(typev[i]));
	}
}

int main(int argc, char **argv)
{
	struct command *command;
	
	if (argc > 1) {
		host = argv[1];
	}
	if (argc > 2) {
		port = atoi(argv[2]);
	}
	printf("Command list from %s:%d\n", host, port);

	command = command_open(host, port);
	command->handler_list_entry = handler_list_entry;

	while (command_state_get(command) != COMMAND_STATE_READY &&
	    command_state_get(command) != COMMAND_STATE_DISCONNECTED) {
		fd_set fdrx;
		int high = 0;
		
		FD_ZERO(&fdrx);

		command_fd_set(command, &fdrx, &high);
		
		select(high + 1, &fdrx, NULL, NULL, NULL);
		
		command_handle(command, &fdrx);
	}

	return 0;
}
