/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007, 2011
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007, 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <math.h>

#include <fftw3.h>
#include <math.h>

#include <trace/trace.h>

static char *tmpfilename;
static FILE *fdplot = NULL;
static FILE *fdtmp = NULL;
static char *trace_name = "?";
static char *trace_unit = "?";

float freq = 250.0;
int fftlen = 10000;
int measure = 10;

int in_len = 0;
fftw_complex *in, *out;
double inac;

void handle_val(struct traceval *trc)
{
	int i;
	float val = trace_val(trc);
	
	if (in_len == 0)
		inac = 0;
	in[in_len][0] = val;
	inac += val;
	// * 0.5 * ( 1.0 - cos( M_PI / fftlen * in_len ));
	in_len++;

	if (in_len == fftlen) {
		inac /= fftlen;
		for (i = 0; i < fftlen; i++) {
			in[in_len][0] -= inac;
			in[in_len][0] *= 1.0 - cos(M_PI / fftlen * i);
		}
		fftw_plan p;
	
		in_len = 0;
		
		p = fftw_plan_dft_1d(fftlen, in, out, FFTW_FORWARD, FFTW_ESTIMATE);

		fftw_execute(p);
		fftw_destroy_plan(p);
		
		freopen(NULL, "w", fdtmp);
		for (i = 0; i < fftlen / 2; i++) {
			fprintf(fdtmp, "%e %e\n",
			    (float)i * freq/fftlen,
			    sqrt(out[i][0]*out[i][0] + out[i][1]*out[i][1]));
		}
		fprintf(fdplot,
		    "set logscale x\n"
		    "plot [%f:%f]"
		    "'%s' using ($1):(20*log($2)) "
		    "title '%s [%s] spectrum' with lines\n",
		    freq/fftlen, freq/2,
		    tmpfilename, trace_name, trace_unit);
		fflush(NULL);
	}
}

int main(int argc, char **argv)
{
	struct traceval *trc;
	int fdmkstemp;
	
	measure = atoi(argv[2]);

	trc = trace_create();
	if (!trc)
		return -1;
        
	tmpfilename = malloc(100);
	sprintf(tmpfilename, "trace_fft_XXXXXX");
	fdmkstemp = mkstemp(tmpfilename);

	fdtmp = fdopen(fdmkstemp, "w");
	if (fdtmp == NULL) {
		perror("fopen() failed");
		return 1;
	}

	fdplot = popen("gnuplot", "w");
	if (fdplot == NULL) {
		perror("popen() failed");
		return 1;
	}
	
	trace_name = argv[1];

	trace_init("localhost", 10000, trace_name, trc);

	freq = 1.0 / trace_period(trc);
	fftlen = freq * measure;
	in = malloc(sizeof(fftw_complex)*fftlen);
	out = malloc(sizeof(fftw_complex)*fftlen);

	trace_handler_add(trc, handle_val);

	while (!trace_handle(trc));

	unlink(tmpfilename);

	return 0;
}
