/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <glib.h>
#include <stdbool.h>
#include <string.h>
#include "dt_host.h"

static char *controller_host = "localhost";
static char *console_host = "localhost";
static char *log_path = "/var/log/dt/";
static bool init_done = false;

static int load_config(void)
{
	GKeyFile *keyfile;
	gboolean loaded = false;
	gchar *keyfilename[] ={ "/etc/dt/dt_host.ini", "dt_host.ini" };
	int ret = 0;
	int i;

	keyfile = g_key_file_new();
	
	for (i = 0; i < sizeof(keyfilename)/sizeof(keyfilename[0]) && !loaded; i++) {
		loaded = g_key_file_load_from_file(
		    keyfile, 
		    keyfilename[i],
		    G_KEY_FILE_NONE,
		    NULL);
	}

	if (loaded) {
		gsize groups_nr;
		gchar **groups;
		int i;
		
		groups = g_key_file_get_groups(keyfile, &groups_nr);
		
		for (i = 0; i < groups_nr; i++) {
			if (!strcmp(groups[i], "controller")) {
				if (g_key_file_has_key(keyfile, groups[i], "host", NULL)) {
					controller_host =
					    g_key_file_get_string(keyfile, groups[i], "host", NULL);
				}
			}
			if (!strcmp(groups[i], "console")) {
				if (g_key_file_has_key(keyfile, groups[i], "host", NULL)) {
					console_host =
					    g_key_file_get_string(keyfile, groups[i], "host", NULL);
				}
			}
			if (!strcmp(groups[i], "log")) {
				if (g_key_file_has_key(keyfile, groups[i], "path", NULL)) {
					log_path =
					    g_key_file_get_string(keyfile, groups[i], "path", NULL);
				}
			}
		}
		
		init_done = true;
		g_strfreev(groups);
	} else {
		ret = -1;;
	}
	g_key_file_free(keyfile);
	
	return ret;
}

char *dt_host_console(void)
{
	if (!init_done)
		load_config();
		
	return console_host;
}

char *dt_host_controller(void)
{
	if (!init_done)
		load_config();
		
	return controller_host;
}

char *dt_host_log_path(void)
{
	if (!init_done)
		load_config();
		
	return log_path;
}

