/*
	tcp_listen.c
	
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/types.h>

#include "tcp_listen.h"

int tcp_listen(int port, int local, int backlog)
{
	int fdlisten, optval;
	struct sockaddr_in sin;

	if ((fdlisten = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
		perror("Can't open socket");
		return -1;
	}

	memset(&sin, 0, sizeof(struct sockaddr_in));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);
	if (local) {
		sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
	} else {
		sin.sin_addr.s_addr = htonl(INADDR_ANY);
	}

	optval = 1;
	setsockopt(fdlisten, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof(optval));
	if (bind(fdlisten,(struct sockaddr*)&sin, sizeof(struct sockaddr_in))==-1) {
		perror("Can't bind socket");
		close(fdlisten);
		return -1;
	}

	if (listen(fdlisten, backlog) == -1) {
		perror("Can't listen on trx socket");
		close(fdlisten);
		return -1;
	}

	optval = 1;
	ioctl(fdlisten, FIONBIO, &optval);

	return fdlisten;
}

int tcp_accept(int fd_listen)
{
	struct sockaddr_in sin;
	socklen_t addrlen = sizeof(struct sockaddr_in);
	int fd;
	int optval;
	
	if ((fd = accept(fd_listen, (struct sockaddr *)&sin, &addrlen)) >= 0) {
		optval = 1;
		ioctl(fd, FIONBIO, &optval);
	}
	
	return fd;
}
