/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007, 2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <math.h>
#include <sys/select.h>
#include <time.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>

#include <command/command.h>


char *command_host = "localhost";
int command_port = 11000;

struct command *sp_command = NULL;


int main(int argc, char **argv)
{
	printf("Controller command shell...\n");

	if (argc < 2) {
		printf("Usage: %s [spg name]\n", argv[0]);
		return 1;
	}
	
	signal(SIGPIPE, SIG_IGN);

	printf("Commands:\n\n");
	printf("setpoint [val] (degrees are converted to radials)\n");
	printf("speed [val]\n");
	printf("setpointtime [val] [time] (degrees are converted to radials)\n");
	printf("\n");

	sp_command = command_open_simple(command_host, command_port, argv[1],
	    "console/command_shell", COMMAND_VALUE_TYPE_FLOAT);
	if (!sp_command) {
		printf("Setpoint generator %s does not exist\n", argv[1]);
		return 1;
	}
	
	printf("Starting main loop.\n");

	do {
		char buffer[100];
		float setpoint;
		uint32_t deadline;
		char *cmd;
		char *value;
		struct command_entry entry;
		
		fgets(buffer, 100, stdin);
		cmd = strtok(buffer, " \t");
		value = strtok(NULL, " \t");
		
		if (!strcmp(cmd, "setpoint")) {
			setpoint = atof(value);
		
			setpoint *= M_PI / 180.0;
			
			entry.type = COMMAND_PTYPE_SETPOINT;
			entry.value.f = setpoint;
			command_send(sp_command, &entry);
		
			printf("Setpoint: %e\n", setpoint);
		}
		if (!strcmp(cmd, "speed")) {
			setpoint = atof(value);
		
			entry.type = COMMAND_PTYPE_SPEED;
			entry.value.f = setpoint;
			command_send(sp_command, &entry);

			printf("Speed: %e\n", setpoint);
		}
		if (!strcmp(cmd, "setpointtime")) {
			char *dl;
			
			dl = strtok(NULL, " \t");
			setpoint = atof(value);
			setpoint *= M_PI / 180.0;
			deadline = atoi(dl);
			deadline += time(NULL);

			entry.type = COMMAND_PTYPE_SETPOINT_TIME;
			entry.value.f = setpoint;
			entry.t.tv_sec = deadline;
			entry.t.tv_nsec = 0;
			command_send(sp_command, &entry);	

			printf("Setpoint: %e Time: %d\n", setpoint, deadline);
		}
		
	} while (1);

	return 0;
}
