/*
	Command generator for 'idle' state

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2008, 2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2008

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <signal.h>

#include <command/command.h>

char *command_host = "localhost";
int command_port = 11000;
char *az_command_spg = "Azimuth_Setpoint";
char *el_command_spg = "Elevation_Setpoint";

int main(int argc, char **argv)
{
	struct command *sp_command_az = NULL;
	struct command *sp_command_el = NULL;

	signal(SIGPIPE, SIG_IGN);

	if (argc > 1) {
		command_host = argv[1];
	}

	do {
		sp_command_az = command_open_simple(command_host, command_port,
		    az_command_spg, "console/idle", COMMAND_VALUE_TYPE_FLOAT);
		if (!sp_command_az) {
			printf("Could not open connection for az commands\n");
			sleep(1);
		}
	} while (!sp_command_az);
	do {
		sp_command_el = command_open_simple(command_host, command_port,
		    el_command_spg, "console/idle", COMMAND_VALUE_TYPE_FLOAT);
		if (!sp_command_el) {
			printf("Could not open connection for el commands\n");
			sleep(1);
		}
	} while (!sp_command_el);

	while (1)
	{
		struct command_entry entry;
		printf("Sending idle command\n");
		
		entry.type = COMMAND_PTYPE_SPEED;
		entry.value.f = 0.0;
		command_send(sp_command_az, &entry);
		command_send(sp_command_el, &entry);

		sleep (1);
	}

}
