/* 
	Javascript for j2000 tracker control 

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2008, 2009, 2011, 2013,
	2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2008, 2009,
	2011, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

var j2000_loaded;
if (!j2000_loaded) {
j2000_loaded = true;

/*
	Communication with the tracker
 */

eval(load("dt_ui.js"));
eval(load("status.js"));

function j2000_command (url, ident)
{
	this.ident = ident;

	this.open = function (ra, dec, switches)
	{
		dt_websocket_send("command " + ident + " " +
		    ra + " " + dec + " " + switches);
	}
}


function j2000_status_cb_default (ra_set, dec_set, ra_now, dec_now, ut_now, switches)
{
	/* default callback */
}

function j2000_status(url, ident)
{
	var j2000_status_this = this;

	/* Callbacks */
	this.callback = j2000_status_cb_default;

	this.status = new status(url, ident);

	this.handle_xml = function j2000_status_handle_xml(data)
	{
		var set_ra;
		var set_dec;
		var now_ra;
		var now_dec;
		var now_ut;
		var switches;
		
		set_ra = data.split(' ')[0];
		set_dec = data.split(' ')[1];
		now_ra = data.split(' ')[2];
		now_dec = data.split(' ')[3];
		now_ut = data.split(' ')[4];
		switches = data.split(' ')[5];
		j2000_status_this.callback(set_ra, set_dec, 
		    now_ra, now_dec, now_ut,
		    switches);
	}

	this.url = url;
	this.ident = ident;

	this.open = function j2000_status_open()
	{
		this.status.callback = this.handle_xml;
		this.status.open();
	}
}



var j2000_status_url = "status.cgi";
var j2000_command_url = "command.cgi_j2000";


function j2000(element_name) {
	var j2000_this = this;

	/* 

		Create all the UI elements

	 */

	this.window = new dt_ui_window(element_name);
	
	this.window.title_set("J2000 tracker");

	this.elem_filler = new dt_ui_element(element_name + "_filler");
	this.lbl_ra = new dt_ui_element(element_name + "_lbl_ra");
	this.lbl_dec = new dt_ui_element(element_name + "_lbl_dec");
	this.lbl_pos_cur = new dt_ui_element(element_name + "_lbl_pos_cur");
	this.lbl_sp_cur = new dt_ui_element(element_name + "_lbl_sp_cur");
	this.lbl_sp_new = new dt_ui_element(element_name + "_lbl_sp_new");
	this.val_ra_cur = new dt_ui_value(element_name + "_val_ra_cur");
	this.val_dec_cur = new dt_ui_value(element_name + "_val_dec_cur");
	this.val_ra_sp = new dt_ui_value(element_name + "_val_ra_sp");
	this.val_dec_sp = new dt_ui_value(element_name + "_val_dec_sp");
	this.txt_ra_sp = new dt_ui_text(element_name + "_txt_ra_sp");
	this.txt_dec_sp = new dt_ui_text(element_name + "_txt_dec_sp");

	this.button_enabled = new dt_ui_button_indicator(element_name + "_enabled");
	this.button_dt_model_enabled = new dt_ui_button_indicator(element_name + "_dt_model_enabled");
	this.button_refraction = new dt_ui_button_indicator(element_name + "_dt_model_refraction");

	this.button_track = new dt_ui_button(element_name + "_track");
	this.button_copy_pos = new dt_ui_button(element_name + "_copy_pos");
	this.button_copy_sp = new dt_ui_button(element_name + "_copy_sp");

	this.button_preset_0 = new dt_ui_button(element_name + "_preset_0");
	this.button_preset_1 = new dt_ui_button(element_name + "_preset_1");
	this.button_preset_2 = new dt_ui_button(element_name + "_preset_2");

	this.window.add([
	    this.elem_filler, this.lbl_ra, this.lbl_dec,
	    this.lbl_pos_cur, this.val_ra_cur, this.val_dec_cur,
	    this.lbl_sp_cur, this.val_ra_sp, this.val_dec_sp,
	    this.lbl_sp_new, this.txt_ra_sp, this.txt_dec_sp,
	    this.button_enabled, this.button_dt_model_enabled, this.button_refraction,
	    this.button_track, this.button_copy_pos, this.button_copy_sp,
	    this.button_preset_0, this.button_preset_1, this.button_preset_2]);

	this.lbl_ra.text_set("Right ascension");
	this.lbl_dec.text_set("Declination");
	this.lbl_pos_cur.text_set("Current position");
	this.lbl_sp_cur.text_set("Current setpoint");
	this.lbl_sp_new.text_set("New setpoint");
	
	this.val_ra_cur.value_set("??");
	this.val_dec_cur.value_set("??");
	this.val_ra_sp.value_set("??");
	this.val_dec_sp.value_set("??");
	/* -ddddmmmss.s = 12*/
	this.txt_ra_sp.length_set(12);
	this.txt_dec_sp.length_set(12);

	this.button_track.text_set("Track new setpoint"); 
	this.button_copy_pos.text_set("Copy current position");
	this.button_copy_sp.text_set("Copy current setpoint");

	this.button_preset_0.text_set("Cassiopeia A");
	this.button_preset_1.text_set("Cygnus A");
	this.button_preset_2.text_set("PSR B0329+54");

	this.window.resize_equal([
	    this.elem_filler, this.lbl_ra, this.lbl_dec,
	    this.lbl_pos_cur, this.val_ra_cur, this.val_dec_cur,
	    this.lbl_sp_cur, this.val_ra_sp, this.val_dec_sp,
	    this.lbl_sp_new, this.txt_ra_sp, this.txt_dec_sp]);
	
	this.window.align_vertical([
	    this.elem_filler, this.lbl_pos_cur, this.lbl_sp_cur, this.lbl_sp_new], 5);

	this.window.align_horizontal([
	    this.elem_filler, this.lbl_ra, this.lbl_dec], 5);
	this.window.align_horizontal([
	    this.lbl_pos_cur, this.val_ra_cur, this.val_dec_cur], 5);
	this.window.align_horizontal([
	    this.lbl_sp_cur, this.val_ra_sp, this.val_dec_sp], 5);
	this.window.align_horizontal([
	    this.lbl_sp_new, this.txt_ra_sp, this.txt_dec_sp], 5);

	this.button_enabled.text_set("Tracking enabled");
	this.button_dt_model_enabled.text_set("DT model");
	this.button_refraction.text_set("Refraction");
	
	this.window.align_vertical([
	    this.lbl_sp_new, 
	    this.button_track], 10);
	this.window.align_vertical([
	    this.button_track,
	    this.button_copy_pos], 5);
	this.window.align_vertical([
	    this.button_copy_pos,
	    this.button_enabled], 10);
	
	this.window.align_horizontal([
	    this.button_copy_pos, this.button_copy_sp], 10);

	this.window.align_horizontal(
	    [this.button_enabled, 
	     this.button_refraction,
	     this.button_dt_model_enabled], 10);

	this.window.resize_equal([
	    this.button_preset_0, this.button_preset_1, this.button_preset_2]);
	
	this.window.align_horizontal([this.lbl_dec, this.button_preset_0], 10);
	this.window.align_vertical([
	    this.button_preset_0, this.button_preset_1, this.button_preset_2]);
	
	/*
	
		block behaviour.
	
	 */
	
	this.pairlist_prev = "??";
	this.enabled = false;
	this.refraction = false;
	this.dt_model_enabled = false;

	this.ra_set = "?";
	this.dec_set = "?";
	this.ra_pos = "?";
	this.dec_pos = "?";
	this.stat_time = 0;

	this.status = new status(j2000_status_url, "j2000");
	this.status.callback = function(stat)
	{
		var i;
		var pairlist;
		var args = stat.split(" ");
		
		j2000_this.ra_set = args[0];
		j2000_this.dec_set = args[1];
		j2000_this.ra_cur = args[2];
		j2000_this.dec_cur = args[3];
		j2000_this.stat_time = args[4];

		j2000_this.val_ra_cur.value_set(j2000_this.ra_cur);
		j2000_this.val_dec_cur.value_set(j2000_this.dec_cur);
		j2000_this.val_ra_sp.value_set(j2000_this.ra_set);
		j2000_this.val_dec_sp.value_set(j2000_this.dec_set);
		
		pairlist = args[5].split(',');
		for (i = 0; i < pairlist.length; i++)
			pairlist[i] = pairlist[i].split('=');
	
		if (!array_compare(pairlist, j2000_this.pairlist_prev)) {
			var i;

			j2000_this.pairlist_prev = pairlist;

			for (i = 0; i < pairlist.length; i++) {
				var boolvalue;

				if (pairlist[i][1].charAt(0) == '0') {
					boolvalue = false;
				} else {
					boolvalue = true;
				}

				if (pairlist[i][0] == 'dt_model') {
					j2000_this.button_dt_model_enabled.color_set(
					    boolvalue ?
					    dt_ui_color_indicator_green :
					    dt_ui_color_indicator_red);
					j2000_this.dt_model_enabled = boolvalue;
				} else if (pairlist[i][0] == 'enabled') {
					j2000_this.button_enabled.color_set(
					    boolvalue ?
					    dt_ui_color_indicator_green :
					    dt_ui_color_indicator_red);
					j2000_this.enabled = boolvalue;
				} else if (pairlist[i][0] == 'refraction') {
					j2000_this.button_refraction.color_set(
					    boolvalue ?
					    dt_ui_color_indicator_green :
					    dt_ui_color_indicator_red);
					j2000_this.refraction = boolvalue;
				}
			}
		}
	}

	this.new_sp_set = function(ra, dec)
	{
		this.txt_ra_sp.value_set(ra);
		this.txt_dec_sp.value_set(dec);
	}

	this.button_preset_0.onclick = function() { 
		j2000_this.new_sp_set('23h23m26.0','058d48m00.0'); }
	this.button_preset_1.onclick = function() { 
		j2000_this.new_sp_set('19h59m28.3565','40d40m02.099'); }
	this.button_preset_2.onclick = function() { 
		j2000_this.new_sp_set('03h32m59.4','054d34m43.6'); }

	this.copy_position = function() {
		j2000_this.new_sp_set(j2000_this.ra_cur, j2000_this.dec_cur);
	}
	this.button_copy_pos.onclick = function() { j2000_this.copy_position(); }

	this.button_copy_sp.onclick = function() {
		j2000_this.new_sp_set(j2000_this.ra_set, j2000_this.dec_set);
	}

	this.command = new command(j2000_command_url, "j2000");

	this.set_track = function(keyvalue)
	{
		j2000_this.command.send(
		    j2000_this.ra_set + " " +
		    j2000_this.dec_set + " " +
		    keyvalue);	
	}

	this.button_track.onclick = function()
	{
		j2000_this.command.send(
		    j2000_this.txt_ra_sp.value_get() + " " +
		    j2000_this.txt_dec_sp.value_get() + " " +
		    "enabled=1");
	}
	
	this.button_enabled.onclick = function(elem) {
		j2000_this.set_track("enabled=" + (j2000_this.enabled ? "0" : "1"));
	}

	this.button_dt_model_enabled.onclick = function(elem) {
		j2000_this.set_track("dt_model=" + (j2000_this.dt_model_enabled ? "0" : "1"));
	}

	this.button_refraction.onclick = function(elem) {
		j2000_this.set_track("refraction=" + (j2000_this.refraction ? "0" : "1"));
	}

	/*
	
		The complete 'j2000' block can be switched on and off.
	
	 */

	this.visible = true;
	this.visible_set = function(visible)
	{
		if (j2000_this.visible != visible && visible) {
			j2000_this.copy_position();
		}
		j2000_this.visible = visible;
		j2000_this.window.visible_set(visible);
		if (!visible) {
			j2000_this.set_track("enabled=0");
		}
	}

	this.status.open();
}

} /* j2000_loaded */
