/* 
	Javascript for DT model 

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2014

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

var model_loaded;
if (!model_loaded) {
model_loaded = true;

eval(load("dt_ui.js"));
eval(load("status.js"));

function model(element_name) {
	var model_this = this;

	/* 

		Create all the UI elements

	 */

	this.window = new dt_ui_window_hide(element_name);
	
	this.window.title_set("DT model");

	this.lbl_name = new dt_ui_element(element_name + "_lbl_name");
	this.lbl_a0 = new dt_ui_element(element_name + "_lbl_a0");
	this.lbl_c1 = new dt_ui_element(element_name + "_lbl_c1");
	this.lbl_c2 = new dt_ui_element(element_name + "_lbl_c2");
	this.lbl_e0 = new dt_ui_element(element_name + "_lbl_e0");
	this.lbl_b = new dt_ui_element(element_name + "_lbl_b");
	this.lbl_za = new dt_ui_element(element_name + "_lbl_za");
	this.lbl_aa = new dt_ui_element(element_name + "_lbl_aa");

	this.val_name = new dt_ui_value(element_name + "_val_name");
	this.val_a0 = new dt_ui_value(element_name + "_val_a0");
	this.val_c1 = new dt_ui_value(element_name + "_val_c1");
	this.val_c2 = new dt_ui_value(element_name + "_val_c2");
	this.val_e0 = new dt_ui_value(element_name + "_val_e0");
	this.val_b = new dt_ui_value(element_name + "_val_b");
	this.val_za = new dt_ui_value(element_name + "_val_za");
	this.val_aa = new dt_ui_value(element_name + "_val_aa");

	this.button_reload = new dt_ui_button(element_name + "_reload");

	this.window.add([
	    this.lbl_name, this.lbl_a0, this.lbl_c1, this.lbl_c2,
	    this.lbl_e0, this.lbl_b, this.lbl_za, this.lbl_aa,
	    this.val_name, this.val_a0, this.val_c1, this.val_c2,
	    this.val_e0, this.val_b, this.val_za, this.val_aa,
	    this.button_reload]);

	this.lbl_name.text_set("Model name");
	this.lbl_a0.text_set("parameter a0");
	this.lbl_c1.text_set("parameter c1");
	this.lbl_c2.text_set("parameter c2");
	this.lbl_e0.text_set("parameter e0");
	this.lbl_b.text_set("parameter b");
	this.lbl_za.text_set("parameter za");
	this.lbl_aa.text_set("parameter aa");

	this.val_name.value_set("?Unknown name?");
	this.val_a0.value_set("??.????????");
	this.val_c1.value_set("??");
	this.val_c2.value_set("??");
	this.val_e0.value_set("??");
	this.val_b.value_set("??");
	this.val_za.value_set("??");
	this.val_aa.value_set("??");
	
	this.button_reload.text_set("Reload");
	
	this.window.resize_equal([
	    this.lbl_name, this.lbl_a0, this.lbl_c1, this.lbl_c2,
	    this.lbl_e0, this.lbl_b, this.lbl_za, this.lbl_aa,
	    this.val_name, this.val_a0, this.val_c1, this.val_c2,
	    this.val_e0, this.val_b, this.val_za, this.val_aa]);

	this.window.align_horizontal([this.lbl_name, this.val_name], 5);
	this.window.align_vertical([
	    this.lbl_name, this.lbl_a0, this.lbl_c1, this.lbl_c2,
	    this.lbl_e0, this.lbl_b, this.lbl_za, this.lbl_aa], 5);
	this.window.align_horizontal([this.lbl_a0, this.val_a0], 5);
	this.window.align_horizontal([this.lbl_c1, this.val_c1], 5);
	this.window.align_horizontal([this.lbl_c2, this.val_c2], 5);
	this.window.align_horizontal([this.lbl_e0, this.val_e0], 5);
	this.window.align_horizontal([this.lbl_b, this.val_b], 5);
	this.window.align_horizontal([this.lbl_za, this.val_za], 5);
	this.window.align_horizontal([this.lbl_aa, this.val_aa], 5);

	this.window.align_vertical([this.lbl_aa, this.button_reload], 10);


	this.status = new status("dummy mode", "dt_model");
	this.vars = "?";
	this.status.callback = function DT_model_status_cb(vars)
	{
		if (vars == model_this.vars)
			return;
		
		var vars_array = vars.split(',');
		var i;

		model_this.vars = vars;
		
		for (i = 0; i < vars_array.length; i++) {
			var varname = vars_array[i].split('=')[0];
			var varvalue = vars_array[i].split('=')[1];

			if (varname == 'name') {
				model_this.val_name.value_set(varvalue);
			}
			if (varname == 'a0') {
				model_this.val_a0.value_set(varvalue);
			}
			if (varname == 'c1') {
				model_this.val_c1.value_set(varvalue);
			}
			if (varname == 'c2') {
				model_this.val_c2.value_set(varvalue);
			}
			if (varname == 'e0') {
				model_this.val_e0.value_set(varvalue);
			}
			if (varname == 'b') {
				model_this.val_b.value_set(varvalue);
			}
			if (varname == 'za') {
				model_this.val_za.value_set(varvalue);
			}
			if (varname == 'aa') {
				model_this.val_aa.value_set(varvalue);
			}
		}
	}

	this.status.open();

	this.command = new command("dummy_dt_command", "dt_model");

	this.button_reload.onclick = function(elem )
	{
		model_this.command.send("reload=1")
	}

	this.window.hide();
}

} /* model_loaded */
