/* 
	Javascript for satellite tracker control 

	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2014
	Copyright Stichting C.A. Muller Radioastronomiestation, 2014

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

var sat_loaded;
if (!sat_loaded) {
sat_loaded = true;

/*
	Communication with the tracker
 */

eval(load("dt_ui.js"));
eval(load("dt_view.js"));
eval(load("status.js"));

var sat_status_url = "status.cgi";
var sat_command_url = "command.cgi_sat";

function sat(element_name) {
	var sat_this = this;

	/* 

		Create all the UI elements

	 */

	this.window = new dt_ui_window(element_name);
	
	this.window.title_set("Satellite tracker");

	this.lbl_current = new dt_ui_element(element_name + "_lbl_current");
	this.val_satname = new dt_ui_value(element_name + "_val_satname");
	this.val_tle_cur = new dt_ui_value(element_name + "_val_tle_cur");

	this.lbl_new = new dt_ui_element(element_name + "_lbl_new");
	this.select_sat = new dt_ui_select(element_name + "_select_sat");
	this.txt_tle_new = new dt_ui_textarea(element_name + "_txt_tle_new");

	this.button_set_tle = new dt_ui_button(element_name + "_set_tle");
	this.button_update_tle = new dt_ui_button(element_name + "_update_tle");

	this.button_enabled = new dt_ui_button_indicator(element_name + "_enabled");
	this.button_dt_model_enabled = new dt_ui_button_indicator(element_name + "_dt_model_enabled");
	this.button_refraction = new dt_ui_button_indicator(element_name + "_dt_model_refraction");

	this.sat_view = new dt_ui_canvas(element_name + "_sat_view");
	this.val_aos = new dt_ui_value(element_name + "_val_aos");
	this.val_los = new dt_ui_value(element_name + "_val_los");

	this.window.add([
	    this.lbl_current, this.val_satname, this.val_tle_cur,
	    this.lbl_new, this.select_sat, this.txt_tle_new,
	    this.button_set_tle, this.button_update_tle,
	    this.button_enabled, this.button_dt_model_enabled, this.button_refraction,
	    this.sat_view, this.val_aos, this.val_los]);

	this.button_enabled.text_set("Tracking enabled");
	this.button_dt_model_enabled.text_set("DT model");
	this.button_refraction.text_set("Refraction");

	this.lbl_current.text_set("Current TLE");
	this.val_satname.value_set("------------------------");

	this.val_tle_cur.wrap_set(true, true);
	this.val_tle_cur.value_set(
	    "---------------------------------------------------------------------" + "\n" +
	    "---------------------------------------------------------------------");

	this.lbl_new.text_set("New TLE");
	this.txt_tle_new.text_height_set(3);
	this.txt_tle_new.text_length_set(69);
	
	this.select_sat.option_add("????????????????????????", null);
	
	this.button_set_tle.text_set("Set new TLE");
	this.button_update_tle.text_set("Update TLE from Celestrak");
	
	this.sat_view.width_set(100);
	this.sat_view.height_set(100);
	this.satellite_view = new satellite_view(this.sat_view.name);
	this.satellite_view.draw(null);
	
	this.val_aos.value_set("??:??:?? UTC");
	this.val_los.value_set("??:??:?? UTC");
	this.window.resize_equal_width([
	    this.sat_view, this.val_aos, this.val_los]);
	this.sat_view.height_set(this.sat_view.width_get());
	    
	this.window.resize_equal([
	    this.val_satname, this.lbl_current, this.lbl_new, this.select_sat]);
	
	this.window.align_horizontal([
	    this.lbl_current, this.val_tle_cur, this.sat_view], 10);

	this.window.align_vertical([
	    this.lbl_current, this.val_satname, this.lbl_new,
	    this.select_sat,
	    this.button_set_tle,
	    this.button_enabled], 10);

	this.window.align_horizontal([
	    this.lbl_new, this.txt_tle_new], 10);

	this.window.align_horizontal([
	    this.button_set_tle, this.button_update_tle], 10);

	this.window.align_horizontal([
	    this.button_enabled, 
	    this.button_refraction,
	    this.button_dt_model_enabled], 10);

	
	this.window.align_vertical([
	    this.sat_view, this.val_aos, this.val_los]);
	
	/*
	
		block behaviour.
	
	 */

	this.pairlist_prev = "??";
	this.enabled = false;
	this.dt_model_enabled = false;
	this.refraction = false;
	this.tle_cur = ["-","-"];

	this.status = new status(sat_status_url, "sat");
	this.status.callback_keyvalue = function(pairlist)
	{
		if (!array_compare(pairlist, sat_this.pairlist_prev)) {
			var i;

			sat_this.pairlist_prev = pairlist;

			for (i = 0; i < pairlist.length; i++) {
				var boolvalue;
				
				if (pairlist[i][1].charAt(0) == '0') {
					boolvalue = false;
				} else {
					boolvalue = true;
				}

				if (pairlist[i][0] == 'dt_model') {
					sat_this.button_dt_model_enabled.color_set(
					    boolvalue ?
					    dt_ui_color_indicator_green :
					    dt_ui_color_indicator_red);
					sat_this.dt_model_enabled = boolvalue;
				} else if (pairlist[i][0] == 'enabled') {
					sat_this.button_enabled.color_set(
					    boolvalue ?
					    dt_ui_color_indicator_green :
					    dt_ui_color_indicator_red);
					sat_this.enabled = boolvalue;
				} else if (pairlist[i][0] == 'refraction') {
					sat_this.button_refraction.color_set(
					    boolvalue ?
					    dt_ui_color_indicator_green :
					    dt_ui_color_indicator_red);
					sat_this.refraction = boolvalue;
				} else if (pairlist[i][0] == 'tle') {
					sat_this.tle_cur[0] =
					    pairlist[i][1].substr(0, 69);
					sat_this.tle_cur[1] =
					    pairlist[i][1].substr(69, 69);
					sat_this.val_tle_cur.value_set(
					    sat_this.tle_cur[0] + "\n" +
					    sat_this.tle_cur[1]);
				} else if (pairlist[i][0] == 'name') {
					sat_this.satname = pairlist[i][1];
					sat_this.val_satname.value_set(
					    sat_this.satname);
				} else if (pairlist[i][0] == 'pass') {
					sat_this.pass_update(pairlist[i][1]);
				}
			}
		}
	}
	
	this.command = new command(sat_command_url, "sat");

	this.set_track = function(keyvalue)
	{
		sat_this.command.send(
		/* "tle=" + tle1 + tle2*/
		    keyvalue);	
	}

	this.button_set_tle.onclick = function()
	{
		var tle = sat_this.txt_tle_new.value_get();
		var command;
		
		command = "tle=" + tle.split('\n').join('');

		sat_this.set_track(command);
	}

	this.button_update_tle.onclick = function()
	{
		var update_ret = load("tle_update.cgi");
		
		alert("Response from console: \n" + update_ret);
		
		sat_this.sat_reload();
	}

	this.button_enabled.onclick = function(elem) {
		sat_this.set_track("enabled=" + (sat_this.enabled ? "0" : "1"));
	}

	this.button_dt_model_enabled.onclick = function(elem) {
		sat_this.set_track("dt_model=" + (sat_this.dt_model_enabled ? "0" : "1"));
	}

	this.button_refraction.onclick = function(elem) {
		sat_this.set_track("refraction=" + (sat_this.refraction ? "0" : "1"));
	}

	this.sat_tle = new Array();

	this.sat_elem_onclick = function(sat_tle_elem)
	{
		sat_this.txt_tle_new.value_set(
		    sat_tle_elem.name + "\n" +
		    sat_tle_elem.tle1 + "\n" +
		    sat_tle_elem.tle2);
//		alert(sat_tle_elem.name);
	}

	this.sat_reload = function()
	{
		var i;
		var amateur;
		var tle_new;
		var gps_ops;
		var element_lines;
		var d = new Date();
		var extra = "?" + d.getTime();
		
		sat_this.select_sat.option_clear();
		sat_this.sat_tle.length = 0;

		amateur = load("tle/amateur.txt" + extra).split('\n');
		tle_new = load("tle/tle-new.txt" + extra).split('\n');
		gps_ops = load("tle/gps-ops.txt" + extra).split('\n');
		
		amateur.length -= amateur.length % 3;
		tle_new.length -= tle_new.length % 3;
		gps_ops.length -= gps_ops.length % 3;
		
		element_lines = amateur.concat(tle_new).concat(gps_ops);
		
		for (i = 0; i < element_lines.length - 2; i += 3) {
			var ti = sat_this.sat_tle.length;
			
			sat_this.sat_tle[ti] = new function() {
				this.name = element_lines[i+0];
				this.tle1 = element_lines[i+1];
				this.tle2 = element_lines[i+2];
			}
		}
		
		sat_this.sat_tle.sort(function(a, b) { return a.name > b.name });
		
		for (i = 0; i < sat_this.sat_tle.length; i++) {		
			sat_this.select_sat.option_add(sat_this.sat_tle[i].name, 
				    sat_this.sat_elem_onclick, sat_this.sat_tle[i]);
		}

	}

	this.sat_reload();

	this.pass_update = function(value)
	{
		var i;
		var coord = value.split(' ');
		var sat_coord = new Array();
	
		for (i = 0; i < coord.length; i++) {
			var t = coord[i].split(':')[0];
			var az = coord[i].split(':')[1];
			var el = coord[i].split(':')[2];
		
			if (t < 1)
				break;
			sat_coord.push( new function() {
				this.t = t;
				this.az = az;
				this.el = el;
			});
			
		}
		
		this.satellite_view.draw(sat_coord);

		var t_aos = new Date();
		var t_los = new Date();

		t_aos.setTime(sat_coord[0].t * 1000);
		t_los.setTime(sat_coord[sat_coord.length - 1].t * 1000);
	
		this.val_aos.value_set(
		    ("0" + t_aos.getUTCHours()).slice(-2) +":"+ 
		    ("0" + t_aos.getUTCMinutes()).slice(-2) +":"+
		    ("0" + t_aos.getUTCSeconds()).slice(-2) + " UTC");
		this.val_los.value_set(
		    ("0" + t_los.getUTCHours()).slice(-2) +":"+
		    ("0" + t_los.getUTCMinutes()).slice(-2) +":"+
		    ("0" + t_los.getUTCSeconds()).slice(-2) + " UTC");
	}
	

	/*
	
		The complete 'sat' block can be switched on and off.
	
	 */


	this.visible_set = function(visible)
	{
		sat_this.window.visible_set(visible);
		if (!visible) {
			sat_this.set_track("enabled=0");
		}
	}

	this.status.open();
}


} /* sat_loaded */
