/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     ---------------------------
     |                         |
  ---| 0  clk       0  counter |----
     |	                       |
     ---------------------------
     
 */

struct controller_block_private {
	bool     *clk;
	bool     prev;
	uint32_t count;
	uint32_t overflow;
};

static void calculate(struct controller_block *counter)
{
	struct controller_block_private *priv = counter->private;
	bool clk = *priv->clk;
	bool prev = priv->prev;

	if (clk	&& !prev)
		priv->count = (priv->count + 1) % priv->overflow;

	priv->prev = clk;
}


static struct controller_block_param_list params[] = {
	{ "overflow", false },
	{ "value", false },
	{ NULL },
};

static void param_get(struct controller_block *counter, int param, void *val)
{
	switch (param) {
		case 0:
			*(int*)val = counter->private->overflow;
			break;
		case 1:
			*(int*)val = counter->private->count;
			break;
	}
}

static void param_set(struct controller_block *counter, int param, va_list val)
{
	switch (param) {
		case 0:
			counter->private->overflow = va_arg(val, int);
			break;
		case 1:
			counter->private->count = va_arg(val, int);
			break;
	}
}
struct controller_block * block_counter_create(char *name)
{
	struct controller_block *counter;
	
	counter = malloc(sizeof(struct controller_block));
	if (!counter)
		return NULL;

	counter->type = "counter";
	counter->name = malloc(strlen(name)+1);
	if (!counter->name)
		goto err_name;
	strcpy(counter->name, name);

	counter->private = malloc(sizeof(struct controller_block_private));
	if (!counter->private)
		goto err_private;
	counter->private->count = 0;

	counter->inputs = 1;
	counter->input = malloc(sizeof(struct controller_block_interm));
	if (!counter->input)
		goto err_inputs;
	counter->input[0].name = "clk";
	counter->input[0].type = CONTROLLER_BLOCK_TERM_BOOL;
	counter->input[0].value.b = &counter->private->clk;
	counter->input[0].ghostof = NULL;

	counter->outputs = 1;
	counter->output = malloc(sizeof(struct controller_block_outterm) * 1);
	if (!counter->output)
		goto err_outputs;
	counter->output[0].name = "counter";
	counter->output[0].type = CONTROLLER_BLOCK_TERM_UINT32;
	counter->output[0].value.u32 = &counter->private->count;
	counter->output[0].source = counter;

	counter->calculate = calculate;

	if (controller_block_param_list_init(counter, params))
		goto err_paramlist;

	counter->param_get = param_get;
	counter->param_set = param_set;

	controller_block_add(counter);
	return counter;

err_paramlist:
	free(counter->output);
err_outputs:
	free(counter->input);
err_inputs:
	free(counter->private);
err_private:
	free(counter->name);
err_name:
	free(counter);
	return NULL;
}
