/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     ----------------------
     |                    |
  ---| 0  in       0  out |----
     |	                  |
     ----------------------
     
 */

struct controller_block_private {
	float *input0;
	float out;
	float gain;
};

static void gain_calculate(struct controller_block *gain)
{
	struct controller_block_private *priv = gain->private;

	priv->out = *priv->input0 * priv->gain;
}

static struct controller_block_param_list params[] = {
	{ "gain", false },
	{ NULL },
};

static void param_get(struct controller_block *gain, int param, void *val)
{
	switch (param) {
		case 0:
			*(float*)val = gain->private->gain;
			break;
	}
}

static void param_set(struct controller_block *gain, int param, va_list val)
{
	switch (param) {
		case 0:
			gain->private->gain = va_arg(val, double);
			break;
	}
}

static struct controller_block_interm_list interms[] = {
	{ "in", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input0) },
	{ NULL }
};

static struct controller_block_outterm_list outterms[] = {
	{ "out", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, out) },
	{ NULL }
};

struct controller_block * block_gain_create(char *name)
{
	struct controller_block *gain;
	
	if (!(gain = controller_block_alloc("gain", name, sizeof(struct controller_block_private))))
		return NULL;

	gain->private->out = 0.0;
	gain->private->gain = 0.0;

	if (controller_block_interm_list_init(gain, interms))
		goto err_block;

	if (controller_block_outterm_list_init(gain, outterms))
		goto err_block;

	gain->calculate = gain_calculate;

	if (controller_block_param_list_init(gain, params))
		goto err_block;

	gain->param_get = param_get;
	gain->param_set = param_set;

	controller_block_add(gain);
	return gain;

err_block:
	controller_block_free(gain);
	return NULL;
}
