/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>


/*
       inputs      outputs
       nr name     nr name

     ----------------------
     |                    |
  ---| 0  in       0  out |----
     |	                  |
     ----------------------
     
 */

struct controller_block_private {
	float *input0;
	float out;
	float min;
	float max;
};

static void limit_calculate(struct controller_block *limit)
{
	struct controller_block_private *priv = limit->private;
	float out;
	float min = priv->min;
	float max = priv->max;

	out = *priv->input0;

	if (out > max)
		out = max;
	if (out < min)
		out = min;
	priv->out = out;
}

static struct controller_block_param_list params[] = {
	{ "min", false },
	{ "max", false },
	{ NULL },
};

static void param_get(struct controller_block *limit, int param, void *val)
{
	switch (param) {
		case 0:
			*(float*)val = limit->private->min;
			break;
		case 1:
			*(float*)val = limit->private->max;
			break;
	}
}

static void param_set(struct controller_block *limit, int param, va_list val)
{
	switch (param) {
		case 0:
			limit->private->min = va_arg(val, double);
			break;
		case 1:
			limit->private->max = va_arg(val, double);
			break;
	}
}

static struct controller_block_interm_list interms[] = {
	{ "in", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input0) },
	{ NULL }
};

static struct controller_block_outterm_list outterms[] = {
	{ "out", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, out) },
	{ NULL }
};

struct controller_block * block_limit_create(char *name)
{
	struct controller_block *limit;
	
	if (!(limit = controller_block_alloc("limit", name, sizeof(struct controller_block_private))))
		return NULL;

	limit->private->out = 0.0;
	limit->private->max = 0.0;
	limit->private->min = 0.0;

	if (controller_block_interm_list_init(limit, interms))
		goto err_block;

	if (controller_block_outterm_list_init(limit, outterms))
		goto err_block;

	limit->calculate = limit_calculate;

	if (controller_block_param_list_init(limit, params))
		goto err_block;

	limit->param_get = param_get;
	limit->param_set = param_set;

	controller_block_add(limit);
	return limit;

err_block:
	controller_block_free(limit);
	return NULL;
}
