/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     ----------------------
     |                    |
  ---| 0  in       0  out |----
     |	                  |
  ---| 1  limit           |
     |	                  |
     ----------------------
     
 */

struct controller_block_private {
	float *input0;
	float *input1;
	float out;
};

static void limit_dyn_calculate(struct controller_block *limit)
{
	struct controller_block_private *priv = limit->private;
	float out;
	float input0 = *priv->input0;
	float input1 = fabs(*priv->input1);

	out = input0;
	if (out > input1)
		out = input1;
	if (out < -input1)
		out = -input1;
	limit->private->out = out;
}

static struct controller_block_interm_list interms[] = {
	{ "in", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input0) },
	{ "limit", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input1) },
	{ NULL }
};

static struct controller_block_outterm_list outterms[] = {
	{ "out", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, out) },
	{ NULL }
};


struct controller_block * block_limit_dyn_create(char *name)
{
	struct controller_block *limit;
	
	if (!(limit = controller_block_alloc("limit_dyn", name, sizeof(struct controller_block_private))))
		return NULL;

	limit->private->out = 0.0;

	if (controller_block_interm_list_init(limit, interms))
		goto err_block;

	if (controller_block_outterm_list_init(limit, outterms))
		goto err_block;
	
	limit->calculate = limit_dyn_calculate;

	limit->params = 0;
	limit->param = NULL;

	limit->param_get = NULL;
	limit->param_set = NULL;

	controller_block_add(limit);
	return limit;

err_block:
	controller_block_free(limit);
	return NULL;
}
