/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     ----------------------
     |                    |
  ---| 0  in       0  out |----
     |	                  |
     ----------------------
     
 */

struct controller_block_private {
	float *input0;
	float out;
	float quantum;
	float invquantum;
};

static void quantize_calculate(struct controller_block *quantize)
{
	struct controller_block_private *priv = quantize->private;
	float input0 = *priv->input0;
	float invquantum = priv->invquantum;
	float quantum = priv->quantum;
	float val;
	
	val = input0 * invquantum;
	val = floorf(val + 0.5);
	priv->out = quantum * val;
}

static struct controller_block_param_list params[] = {
	{ "quantum", true },
	{ NULL },
};

static void param_get(struct controller_block *quantize, int param, void *val)
{
	switch (param) {
		case 0:
			*(float*)val = quantize->private->quantum;
			break;
	}
}

static void param_set(struct controller_block *quantize, int param, va_list val)
{
	switch (param) {
		case 0:
			quantize->private->quantum = va_arg(val, double);
			break;
	}
	quantize->private->invquantum = 1.0 / quantize->private->quantum;
}

static struct controller_block_interm_list interms[] = {
	{ "in", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, input0) },
	{ NULL }
};

static struct controller_block_outterm_list outterms[] = {
	{ "out", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, out) },
	{ NULL }
};

struct controller_block * block_quantize_create(char *name)
{
	struct controller_block *quantize;
	
	if (!(quantize = controller_block_alloc("quantize", name, sizeof(struct controller_block_private))))
		return NULL;

	quantize->private->out = 0.0;
	quantize->private->quantum = 1.0;
	quantize->private->invquantum = 1.0;

	if (controller_block_interm_list_init(quantize, interms))
		goto err_block;

	if (controller_block_outterm_list_init(quantize, outterms))
		goto err_block;

	quantize->calculate = quantize_calculate;

	if (controller_block_param_list_init(quantize, params))
		goto err_block;

	quantize->param_get = param_get;
	quantize->param_set = param_set;

	controller_block_add(quantize);
	return quantize;

err_block:
	controller_block_free(quantize);
	return NULL;
}
