/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <controller/controller_block.h>
#include <controller/controller_sample.h>

/*
  	 outputs
  	 nr name

  -----------------
  |		  |
  |	 0  value |----
  |		  |
  -----------------

 */

struct controller_block_private {
	float value;
	
	float step;
	float current;
	
	float f;
};

static void sine_calculate(struct controller_block *sine)
{
	struct controller_block_private *priv = sine->private;
	float current = priv->current;
	
	current += priv->step;
	if (current > M_PI * 2)
		current -= M_PI * 2;
	priv->value = sinf(current);
	
	priv->current = current;
}

static struct controller_block_param_list params[] = {
	{ "f", false },
	{ NULL },
};

static void param_get(struct controller_block *sine, int param, void *val)
{
	switch (param) {
		case 0:
			*(float*)val = sine->private->f;
			break;
	}
}

static void param_set(struct controller_block *sine, int param, va_list val)
{
	switch (param) {
		case 0:
			sine->private->f = va_arg(val, double);
			break;
	}
	
	sine->private->step = 
	    2 * M_PI * sine->private->f * controller_sample_period();
}

static struct controller_block_outterm_list outterms[] = {
	{ "value", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, value) },
	{ NULL }
};

struct controller_block * block_sine_create(char *name)
{
	struct controller_block *sine;
	
	if (!(sine = controller_block_alloc("sine", name, sizeof(struct controller_block_private))))
		return NULL;

	sine->private->value = 0.0;

	if (controller_block_outterm_list_init(sine, outterms))
		goto err_block;

	sine->calculate = sine_calculate;

	if (controller_block_param_list_init(sine, params))
		goto err_block;

	sine->private->f = 0.0;
	sine->private->current = 0.0;
	sine->private->step = 0.0;

	sine->param_get = param_get;
	sine->param_set = param_set;

	controller_block_add(sine);
	return sine;

err_block:
	controller_block_free(sine);
	return NULL;
}
