/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2012

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
       inputs      outputs
       nr name     nr name

     -----------------------
     |                     |
     |             0     q |----
     |	                   |
     -----------------------
     
 */

struct controller_block_private {
	bool     q;
	bool     trigger;
};

static void trigger_calculate(struct controller_block *trigger)
{
	struct controller_block_private *priv = trigger->private;
	
	priv->q = priv->trigger;
	priv->trigger = false;
}

static struct controller_block_param_list params[] = {
	{ "trigger", false },
	{ NULL },
};

static void param_get(struct controller_block *value, int param, void *val)
{
	/* We have only one parameter... */
	*(int*)val = value->private->trigger;
}

static void param_set(struct controller_block *value, int param, va_list val)
{
	/* we have only one parameter... */
	value->private->trigger = va_arg(val, int);
}

static struct controller_block_outterm_list outterms[] = {
	{ "q", CONTROLLER_BLOCK_TERM_BOOL, offsetof(struct controller_block_private, q) },
	{ NULL }
};

struct controller_block * block_trigger_create(char *name, va_list ap)
{
	struct controller_block *trigger;
	
	if (!(trigger = controller_block_alloc("trigger", name, sizeof(struct controller_block_private))))
		return NULL;

	trigger->private->trigger = false;

	trigger->private->q = false;
	
	trigger->inputs = 0;
	trigger->input = NULL;

	if (controller_block_outterm_list_init(trigger, outterms))
		goto err;

	trigger->calculate = trigger_calculate;

	if (controller_block_param_list_init(trigger, params))
		goto err;

	trigger->param_get = param_get;
	trigger->param_set = param_set;

	controller_block_add(trigger);
	return trigger;

err:
	controller_block_free(trigger);
	return NULL;
}
