/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
  	 outputs
  	 nr name

  -----------------
  |		  |
  |	 0  value |----
  |		  |
  -----------------
 */

struct controller_block_private {
	float value;
};

static struct controller_block_param_list params[] = {
	{ "value", false },
	{ NULL },
};

static void param_get(struct controller_block *value, int param, void *val)
{
	/* We have only one parameter... */
	*(float*)val = value->private->value;
}

static void param_set(struct controller_block *value, int param, va_list val)
{
	/* we have only one parameter... */
	value->private->value = va_arg(val, double);
}

static struct controller_block_outterm_list outterms[] = {
	{ "value", CONTROLLER_BLOCK_TERM_FLOAT, offsetof(struct controller_block_private, value) },
	{ NULL }
};

struct controller_block * block_value_create(char *name)
{
	struct controller_block *value;
	
	if (!(value = controller_block_alloc("value", name, sizeof(struct controller_block_private))))
		return NULL;

	value->private->value = 0.0;
	
	if (controller_block_outterm_list_init(value, outterms))
		goto err_block;

	if (controller_block_param_list_init(value, params))
		goto err_block;

	value->param_get = param_get;
	value->param_set = param_set;

	controller_block_add(value);
	return value;

err_block:
	controller_block_free(value);
	return NULL;
}
