/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2011
	Copyright Stichting C.A. Muller Radioastronomiestation, 2011

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>

/*
  	 outputs
  	 nr name

  -----------------
  |		  |
  |	 0  value |----
  |		  |
  -----------------
 */

struct controller_block_private {
	uint32_t value;
};

static struct controller_block_param_list params[] = {
	{ "value", false },
	{ NULL },
};

static void param_get(struct controller_block *value, int param, void *val)
{
	/* We have only one parameter... */
	*(uint32_t*)val = value->private->value;
}

static void param_set(struct controller_block *value, int param, va_list val)
{
	/* we have only one parameter... */
	value->private->value = va_arg(val, int);
}

struct controller_block * block_value_uint32_create(char *name)
{
	struct controller_block *value;
	
	value = malloc(sizeof(struct controller_block));
	if (!value)
		return NULL;

	value->type = "value_uint32";
	value->name = malloc(strlen(name)+1);
	if (!value->name)
		goto err_value;
	strcpy(value->name, name);

	value->private = malloc(sizeof(struct controller_block_private));
	if (!value->private)
		goto err_name;
	value->private->value = 0.0;
	
	value->inputs = 0;
	value->input = NULL;
	
	value->outputs = 1;
	value->output = malloc(sizeof(struct controller_block_outterm));
	if (!value->output)
		goto err_private;
	value->output[0].name = "value";
	value->output[0].type = CONTROLLER_BLOCK_TERM_UINT32;
	value->output[0].value.u32 = &value->private->value;
	value->output[0].source = value;

	value->calculate = NULL;

	if (controller_block_param_list_init(value, params))
		goto err_output;

	value->param_get = param_get;
	value->param_set = param_set;

	controller_block_add(value);
	return value;

err_output:
	free(value->output);
err_private:
	free(value->private);
err_name:
	free(value->name);
err_value:
	free(value);
	return NULL;
}
