/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2009
	Copyright Stichting C.A. Muller Radioastronomiestation, 2009

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

%{

#include <stdio.h>
#include "il2c.h"

int yylex(void);

%}

%start list

%token PROGRAMSYM

%token VARSYM
%token VAR_CONFIGSYM
%token VAR_INPUTSYM
%token VAR_OUTPUTSYM

%token END_VARSYM

%token COMMASYM
%token LETSYM

%token BOOLSYM
%token UINT8SYM
%token UINT16SYM
%token UINT32SYM
%token UINT64SYM
%token SINT8SYM
%token SINT16SYM
%token SINT32SYM
%token SINT64SYM
%token REAL32SYM
%token REAL64SYM

%token TYPEDLITERALSYM

%token SYSTEM_TIMESYM

%token LDSYM
%token STSYM
%token LDNSYM
%token STNSYM
%token ANDSYM
%token ORSYM
%token ANDNSYM
%token ORNSYM
%token XORSYM
%token ADDSYM
%token SUBSYM
%token MULSYM
%token DIVSYM
%token MODSYM
%token GTSYM
%token EQSYM
%token NESYM
%token LESYM
%token LTSYM
%token SSYM
%token RSYM
%token BEGINPARSYM
%token ENDPARSYM
%token JMPSYM
%token JMPCSYM
%token JMPCNSYM
%token NEGATESYM
%token <string> VARIABLE
%token <string> LITERALSINT
%token <string> LITERALUINT
%token <string> LITERALREAL
%token <string> LABEL

%token ABSSYM
%token SQRTSYM
%token LNSYM
%token LOGSYM 
%token EXPSYM 
%token SINSYM 
%token COSSYM 
%token TANSYM 
%token ASINSYM
%token ACOSSYM
%token ATANSYM
%token EXPTSYM

%token SHLSYM
%token SHRSYM
%token RORSYM
%token ROLSYM
%token NOTSYM

%token MINSYM
%token MAXSYM
%token LIMITSYM
%token SELSYM

%token TYPECONVERSIONSYM
%token TRUNCSYM

%type <string> mathfunc
%type <sym> vartype
%type <string> anyvar
%type <string> literal

%union
{
	char *string;
	int sym;
}

%%

list	: /* empty */
	| item
	| list item
	;

item	: PROGRAMSYM VARIABLE
		{
			il2c_program($2);
		}
	| VARSYM
		{
			il2c_var_category(IL2C_VAR);
		}
		varlist
		{
			il2c_var_category(IL2C_VAR_NONE);
		}
		END_VARSYM
	| VAR_CONFIGSYM
		{
			il2c_var_category(IL2C_VAR_CONFIG);
		}
		varlist
		{
			il2c_var_category(IL2C_VAR_NONE);
		}
		END_VARSYM
	| VAR_INPUTSYM
		{
			il2c_var_category(IL2C_VAR_INPUT);
		}
		varlist
		{
			il2c_var_category(IL2C_VAR_NONE);
		}
		END_VARSYM
	| VAR_OUTPUTSYM
		{
			il2c_var_category(IL2C_VAR_OUTPUT);
		}
		varlist
		{
			il2c_var_category(IL2C_VAR_NONE);
		}
		END_VARSYM
	|	{
			il2c_var_declare();
			il2c_sample_header();
		} 
		instr

instr	: LABEL
		{
			il2c_label($1);
		}
	| LDSYM VARIABLE
		{
			il2c_load($2, false);
		}
	| LDNSYM VARIABLE
		{
			il2c_load($2, true);
		}
	| STSYM VARIABLE
		{ 
			il2c_store($2, false);
		}
	| STNSYM VARIABLE
		{ 
			il2c_store($2, true);
		}
	| SSYM VARIABLE
		{
			il2c_set($2, true);
		}
	| RSYM VARIABLE
		{
			il2c_set($2, false);
		}
	| LDSYM literal
		{
			il2c_load_literal($2, il2c_literal_type_get(), false);
		}
	| LDNSYM literal
		{
			il2c_load_literal($2, il2c_literal_type_get(), true);
		}
	| JMPSYM VARIABLE
		{
			il2c_jump($2);
		}
	| JMPCSYM VARIABLE
		{
			il2c_jumpcond($2, false);
		}
	| JMPCNSYM VARIABLE
		{
			il2c_jumpcond($2, true);
		}
	| ABSSYM
		{
			il2c_abs();
		}
	| EXPTSYM anyvar
		{
			il2c_expt($2);
		}
	| SHRSYM anyvar
		{
			il2c_shift(">>", $2);
		}
	| SHLSYM anyvar
		{
			il2c_shift("<<", $2);
		}
	| RORSYM anyvar
		{
			il2c_rotate($2, false);
		}
	| ROLSYM anyvar
		{
			il2c_rotate($2, true);
		}
	| NOTSYM
		{
			il2c_not();
		}
	| vartype TYPECONVERSIONSYM vartype
		{
			il2c_converttype($1, $3);
		}
	| TRUNCSYM
		{
			il2c_converttype(REAL64SYM, SINT32SYM);
		}
	| MINSYM anyvar
		{
			il2c_minmax(false, $2);
		}
	| MAXSYM anyvar
		{
			il2c_minmax(true, $2);
		}
	| LIMITSYM anyvar COMMASYM anyvar
		{
			il2c_minmax(true, $2);
			il2c_minmax(false, $4);
		}
	| SELSYM anyvar COMMASYM anyvar
		{
			il2c_sel($2, $4, il2c_literal_type_get());
		}
	| mathfunc
		{
			il2c_mathfunc($1);
		}
	|	{
			il2c_op_start();
		} op
		{
			il2c_op_end();
		}
	;
	
anyvar	: VARIABLE
		{
			il2c_literal_type_set(0);
			$$ = $1;
		}
	| literal
		{ $$ = $1; }
	;

mathfunc: SQRTSYM
		{ $$ = "sqrt"; }
	| LNSYM
		{ $$ = "log"; }
	| LOGSYM
		{ $$ = "log10"; }
	| EXPSYM 
		{ $$ = "exp"; }
	| SINSYM 
		{ $$ = "sin"; }
	| COSSYM 
		{ $$ = "cos"; }
	| TANSYM 
		{ $$ = "tan"; }
	| ASINSYM
		{ $$ = "asin"; }
	| ACOSSYM
		{ $$ = "acos"; }
	| ATANSYM
		{ $$ = "atan"; }
	;

oplist	: op 
	| op oplist
	

op	: BEGINPARSYM
		{
			il2c_op_stack_push();
		}
		oplist ENDPARSYM
		{
			il2c_op_stack_pop();
			il2c_op_token("");
		}
	| 	{
			il2c_op_acc_use();
		}
		operator
	| VARIABLE
		{
			il2c_op_usevar($1);
		}
	| literal
		{
			il2c_op_literal($1, il2c_literal_type_get());
		}
	;

operator: ANDSYM
		{
			il2c_op_token(" & ");
		} op
	| ORSYM
		{
			il2c_op_token(" | ");
		} op
	| ANDNSYM
		{
			il2c_op_token(" & !");
		} op
	| ORNSYM
		{
			il2c_op_token(" | !");
		} op
	| XORSYM
		{
			il2c_op_token(" ^ ");
		} op
	| ADDSYM
		{
			il2c_op_token(" + ");
		} op
	| SUBSYM
		{
			il2c_op_token(" - ");
		} op
	| MULSYM
		{
			il2c_op_token(" * ");
		} op
	| DIVSYM
		{
			il2c_op_token(" / ");
		} op
	| MODSYM
		{
			il2c_op_token(" % ");
			il2c_op_token_sym(SINT32SYM);
			il2c_op_result(SINT32SYM);
		} op
	| GTSYM
		{
			il2c_op_token(" > ");
			il2c_op_result(BOOLSYM);
		} op
	| EQSYM
		{
			il2c_op_token(" == ");
			il2c_op_result(BOOLSYM);
		} op
	| NESYM
		{
			il2c_op_token(" != ");
			il2c_op_result(BOOLSYM);
		} op
	| LESYM
		{
			il2c_op_token(" <= ");
			il2c_op_result(BOOLSYM);
		} op
	| LTSYM
		{
			il2c_op_token(" < ");
			il2c_op_result(BOOLSYM);
		} op
	;

varlist	: var
	| varlist var
	;

var	: LABEL vartype
		{
			il2c_var_add($2, $1, 0, NULL);
		}
	| LABEL vartype LETSYM literal
		{
			il2c_var_add($2, $1, il2c_literal_type_get(), $4);
		}
	;

literal	: LITERALSINT
		{
			il2c_literal_type_set(SINT32SYM);
			$$ = $1;
		}
	| LITERALUINT
		{
			il2c_literal_type_set(UINT32SYM);
			$$ = $1;
		}
	| LITERALREAL
		{
			il2c_literal_type_set(REAL64SYM);
			$$ = $1;
		}
	| SYSTEM_TIMESYM
		{
			il2c_literal_type_set(il2c_system_time_type());
			$$ = il2c_system_time();
		}
	| vartype TYPEDLITERALSYM literal
		{
			il2c_literal_type_set($1);
			$$ = $3;
		}
	;

vartype	: SINT8SYM
	  	{
	  		$$ = SINT8SYM;
	  	}
	| SINT16SYM
		{
			$$ = SINT16SYM;
		}
	| SINT32SYM
		{
			$$ = SINT32SYM;
		}
	| SINT64SYM
		{
			$$ = SINT64SYM;
		}
	| UINT8SYM
		{
			$$ = UINT8SYM;
		}
	| UINT16SYM
		{
			$$ = UINT16SYM;
		}
	| UINT32SYM
		{
			$$ = UINT32SYM;
		}
	| UINT64SYM
		{
			$$ = UINT64SYM;
		}
	| BOOLSYM
		{
			$$ = BOOLSYM;
		}
	| REAL32SYM
		{
			$$ = REAL32SYM;
		}
	| REAL64SYM
		{
			$$ = REAL64SYM;
		}
	;

%%

