/*
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_CONTROLLER_BUS_H_
#define _INCLUDE_CONTROLLER_BUS_H_

#include "controller_block.h"

enum controller_bus_state {
	CONTROLLER_BUS_STATE_OK,
	CONTROLLER_BUS_STATE_RECOVERABLE,
	CONTROLLER_BUS_STATE_ERROR,
};

char *controller_bus_state_2_str(enum controller_bus_state);

struct controller_bus_client {
	struct controller_block *block;
	
	void (*callback)(struct controller_block *block, void *data);
	enum controller_bus_state (*poll)(struct controller_bus_client *client);
	int (*recover)(struct controller_bus_client *client);
	
	size_t offset;
	enum controller_bus_state state;

	struct controller_bus_client_private *private;	
};

struct controller_bus {
	char *name;
	char *type;
	
	struct controller_block *owner;
	struct controller_block *input_block;
	struct controller_block *output_block;

	int input_clients;
	struct controller_bus_client *input_client;
	
	int output_clients;
	struct controller_bus_client *output_client;

	enum controller_bus_state (*poll)(struct controller_bus *bus);
	int (*recover)(struct controller_bus *bus);

	enum controller_bus_state state;

	struct controller_bus_private *private;
	struct controller_bus *next;
};

struct controller_bus *controller_bus_create(char *type, char *name,
    struct controller_block *owner,
    size_t private_size);

int controller_bus_input_set(struct controller_bus *bus,
    struct controller_block *block);

int controller_bus_output_set(struct controller_bus *bus,
    struct controller_block *block);

int controller_bus_poll_set(struct controller_bus *bus,
    enum controller_bus_state (*poll)(struct controller_bus *bus),
    int (*recover)(struct controller_bus *bus));
    

struct controller_bus *controller_bus_find(char *name);

struct controller_bus_client *controller_bus_add_input_client(
    struct controller_bus *bus,
    struct controller_block *client);

struct controller_bus_client *controller_bus_add_output_client(
    struct controller_bus *bus,
    struct controller_block *client);

bool controller_bus_linked(struct controller_block *src, 
    struct controller_block *dst);

struct controller_bus *controller_bus_linked_bus(struct controller_block *src, 
    struct controller_block *dst);

struct controller_bus *controller_bus_by_owner(struct controller_block *owner);


int controller_bus_poll_states(int *oks, int *errors, int *recoverables);
int controller_bus_recover(void);

#endif /* _INCLUDE_CONTROLLER_BUS_H_ */
