/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <controller/controller_block.h>
#include <log/log.h>

struct controller_block *ec_block;

struct controller_block_private {
	bool pdo_data;
	bool operational;
};

static void param_set(struct controller_block *ec, int param, va_list val)
{
	int enabled = va_arg(val, int);

	ec->private->pdo_data = enabled;
	ec->private->operational = enabled;
}

static struct controller_block_param_list params[] = {
	{ "pdo_enabled", true },
	{ NULL },
};

static struct controller_block_outterm_list outterms[] = {
	{ "pdo_data",     CONTROLLER_BLOCK_TERM_BOOL, offsetof(struct controller_block_private, pdo_data)  },
	{ "operational",  CONTROLLER_BLOCK_TERM_BOOL, offsetof(struct controller_block_private, operational)  },
	{ NULL },
};

struct controller_block * block_ec_sim_create(char *name, va_list ap)
{
	struct controller_block *ec;
	
	/* For now we only support a single ethercat interface,
	   but we allow multiple inits
	 */
	if (ec_block) {
		return ec_block;
	}

	ec = controller_block_alloc("ec_sim", name, 
	    sizeof(struct controller_block_private));
	if (!ec)
		goto err;
	
	ec->private->pdo_data = true;
	ec->private->operational = true;

	if (controller_block_outterm_list_init(ec, outterms))
		goto err_ec;	

	if (controller_block_param_list_init(ec, params))
		goto err_ec;

	ec->param_get = NULL;
	ec->param_set = param_set;

	controller_block_add(ec);

	ec_block = ec;

	return ec;

err_ec:
	controller_block_free(ec);
err:
	return NULL;
}
