/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2007, 2008
	Copyright Stichting C.A. Muller Radioastronomiestation, 2007, 2008

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_EC_INT_H_
#define _INCLUDE_EC_INT_H_

#include <ec/ec.h>
#include <ec/esc.h>
#include <ec/esc_registers.h>

#define ETH_P_ETHERCAT	0x88a4

/* datagram timeout in usec */
#define EC_DGRAM_TIMEOUT 10000

struct ec_frame_hdr {
	uint16_t len;
} __attribute((__packed__));

#define EC_FRAME_HDR_LEN(hdr) \
     (le16toh((struct ec_frame_hdr *)(hdr)->len) & 0x07ff)
#define EC_FRAME_HDR_TYPE(hdr) \
     ((le16toh((struct ec_frame_hdr *)(hdr)->len) >> 12) & 0x000f)
#define EC_FRAME_HDR_SET(hdr, framelen, type) \
	do { \
     		((struct ec_frame_hdr *)(hdr))->len = \
		    htole16( (framelen) | (type << 12) ); \
	} while (0)

#define EC_FRAME_HDR_TYPE_EC 0x1

struct ec_dgram_hdr {
	uint8_t cmd;
	uint8_t idx;
	union {
		struct { 
			uint16_t pos;
			uint16_t off;
		} position;
		struct {
			uint16_t addr;
			uint16_t off;
		} node;
		struct {
			uint32_t addr;
		} logical;
	} addr;
	uint16_t len;
	uint16_t irq;
} __attribute((__packed__));

#define EC_DGRAM_HDR_LEN(hdr) \
    (le16toh( ((struct ec_dgram_hdr *)(hdr))->len ) & 0x07ff)

#define EC_DGRAM_HDR_C(hdr) \
    ((le16toh( ((struct ec_dgram_hdr *)(hdr))->len ) >> 14) & 0x1)

#define EC_DGRAM_HDR_M(hdr) \
    ((le16toh( ((struct ec_dgram_hdr *)(hdr))->len ) >> 15) & 0x1)

#define EC_DGRAM_HDR_SET(hdr, dlen, c, m) \
	do { \
		((struct ec_dgram_hdr *)(hdr))->len = htole16( \
		    (dlen) | ((c) << 14) | (m) << 15\
		); \
	} while (0)

#define EC_DGRAM_HDR_CMD_NOP	0
#define EC_DGRAM_HDR_CMD_APRD	1
#define EC_DGRAM_HDR_CMD_APWR	2
#define EC_DGRAM_HDR_CMD_APRW	3
#define EC_DGRAM_HDR_CMD_FPRD	4
#define EC_DGRAM_HDR_CMD_FPWR	5
#define EC_DGRAM_HDR_CMD_FPRW	6
#define EC_DGRAM_HDR_CMD_BRD	7
#define EC_DGRAM_HDR_CMD_BWR	8
#define EC_DGRAM_HDR_CMD_BRW	9
#define EC_DGRAM_HDR_CMD_LRD	10
#define EC_DGRAM_HDR_CMD_LWR	11
#define EC_DGRAM_HDR_CMD_LRW	12
#define EC_DGRAM_HDR_CMD_ARMW	13
#define EC_DGRAM_HDR_CMD_FRMW	14


#endif /* _INCLUDE_EC_INT_H_ */
