/*
	Copyright Jeroen Vreeken (pe1rxq@amsat.org), 2013
	Copyright Stichting C.A. Muller Radioastronomiestation, 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#ifndef _INCLUDE_EC_ESC_DEVICE_H_
#define _INCLUDE_EC_ESC_DEVICE_H_

#include <stdbool.h>

#include <controller/controller_bus.h>

#include <ec/ec.h>
#include <ec/esc.h>
#include <ec/esc_sm.h>
#include <ec/esc_dc.h>

struct esc_device {
	struct ec_dgram_addr addr;
	
	struct esc_esi_category *categories;
	
	struct esc_syncmanager_info sm[ESC_SYNCMANAGER_MAX];
	bool rx_pdo;
	bool tx_pdo;
	int rx_pdo_sm;
	int tx_pdo_sm;
	bool tx_pdo_watchdog;
	int mbox_rd_sm;
	int mbox_wr_sm;
	
	struct esc_mailbox *mailbox;
	struct esc_dc_sync dc_sync;

	struct {
		size_t logical_start;
		size_t physical_start;
		size_t length;
		bool used;
	} fmmu[2];

	char *name;
	
	enum esc_al_state state;
	enum esc_al_status status;
	
	struct controller_block *block; /* block this device 'belongs' to */
	struct controller_block *block_tx; /* block producing tx pdo data */
	struct controller_block *block_rx; /* block consuming rx pdo data */
	struct controller_bus *bus;
	
	struct controller_bus_client *client_tx;
	struct controller_bus_client *client_rx;
	
	void (*callback_tx)(struct controller_block *block, void *data);
	void (*callback_rx)(struct controller_block *block, void *data);
	
	int (*pre_operational_hook)(struct esc_device *esc);
};


struct esc_device *esc_device_create(char *busname, int devno, char *name);
struct esc_device *esc_device_create2(struct ec_dgram_addr *ec_addr, char *name);
void esc_device_destroy(struct esc_device *);
int esc_device_poll_start(struct esc_device *dev);

int esc_device_initialize_pre_operational(struct esc_device *dev);
int esc_device_initialize_operational(struct esc_device *dev);

#endif /* _INCLUDE_EC_ESC_DEVICE_H_ */
