/*
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <ec/esc_fmmu.h>
#include <ec/block_ec.h>

#include <ec/esc_registers.h>

#include <log/log.h>

int esc_fmmu_map_rx(struct esc_device *dev, size_t start, size_t len)
{
	size_t rx = dev->bus->private->logical_rx;

	if (dev->fmmu[0].used) {
		log_send(LOG_T_WARNING, "%s: RX FMMU alread alloced",
		    dev->name);
		return 0;
	}
	
	dev->fmmu[0].logical_start = rx;
	dev->fmmu[0].physical_start = start;
	dev->fmmu[0].length = len;
	dev->fmmu[0].used = true;

	dev->client_rx->offset = rx - EC_LOGICAL_RX_START;

	log_send(LOG_T_DEBUG, "%s: RX FMMU: 0x%08zx @ 0x%08zx",
	    dev->name, len, rx);
	
	rx += len;
	
	dev->bus->private->logical_rx = rx;


	return 0;	
}

int esc_fmmu_map_tx(struct esc_device *dev, size_t start, size_t len)
{
	size_t tx = dev->bus->private->logical_tx;
	
	if (dev->fmmu[1].used) {
		log_send(LOG_T_WARNING, "%s: TX FMMU alread alloced",
		    dev->name);
		return 0;
	}

	dev->fmmu[1].logical_start = tx;
	dev->fmmu[1].physical_start = start;
	dev->fmmu[1].length = len;
	dev->fmmu[1].used = true;

	dev->client_tx->offset = tx - EC_LOGICAL_TX_START;

	log_send(LOG_T_DEBUG, "%s: TX FMMU: 0x%08zx @ 0x%08zx",
	    dev->name, len, tx);
	
	tx += len;
	
	dev->bus->private->logical_tx = tx;

	return 0;	
}

static int esc_fmmu_set_nr(struct esc_device *dev, int nr, 
    size_t physical_start, size_t length, size_t logical_start)
{
	struct ec_dgram_addr *addr = &dev->addr;
	uint32_t leval32;
	uint16_t leval16;
	uint8_t val8;
	
	leval32 = htole32(logical_start);
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_LOGICAL_START;
	ec_datagram_write(addr, &leval32, 4);

	val8 = 0;
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_LOGICAL_START_BIT;
	ec_datagram_write(addr, &val8, 1);

	val8 = 7;
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_LOGICAL_STOP_BIT;
	ec_datagram_write(addr, &val8, 1);

	leval16 = htole16(length);
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_LENGTH;
	ec_datagram_write(addr, &leval16, 2);

	leval16 = htole16(physical_start);
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_PHYSICAL_START;
	ec_datagram_write(addr, &leval16, 2);

	val8 = 0;
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_PHYSICAL_START_BIT;
	ec_datagram_write(addr, &val8, 1);

	val8 = ESC_FMMU_TYPE_RW;
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_TYPE;
	ec_datagram_write(addr, &val8, 1);

	val8 = ESC_FMMU_ACTIVATE_ACTIVATED;
	addr->addr.position.off = ESC_ADDR_MAP_FMMU_NR(nr) | ESC_ADDR_MAP_FMMU_ACTIVATE;
	ec_datagram_write(addr, &val8, 1);

	return 0;
}

int esc_fmmu_set(struct esc_device *dev)
{
	int r = 0;
	
	if (dev->fmmu[0].used)
		r = esc_fmmu_set_nr(dev, 0,
		    dev->fmmu[0].physical_start,
		    dev->fmmu[0].length,
		    dev->fmmu[0].logical_start);

	if (dev->fmmu[1].used)
		r |= esc_fmmu_set_nr(dev, 1,
		    dev->fmmu[1].physical_start,
		    dev->fmmu[1].length,
		    dev->fmmu[1].logical_start);

	return r;
}
