/*
	Copyright Jeroen Vreeken (jeroen@vreeken.net), 2013

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <ec/esc_device.h>
#include <ec/esc_sm.h>
#include <ec/esc_registers.h>

#include <log/log.h>

int esc_sm_set(struct esc_device *dev, int nr)
{
	struct ec_dgram_addr *addr = &dev->addr;
	uint8_t val8;
	uint16_t leval16;
	struct esc_syncmanager_info *sm = &dev->sm[nr];

	log_send(LOG_T_DEBUG, "%s: sm%d: %zd bytes @ 0x%zx, ctrl: 0x%02x",
	    dev->name, nr, sm->len, sm->start, sm->ctrl);

	addr->addr.position.off = ESC_ADDR_MAP_SYNCMANAGER_NR(nr) |
	   ESC_ADDR_MAP_SYNCMANAGER_ACTIVATE;
	val8 = 0x00;
	ec_datagram_write(addr, &val8, 1);
	
	addr->addr.position.off = ESC_ADDR_MAP_SYNCMANAGER_NR(nr) |
	    ESC_ADDR_MAP_SYNCMANAGER_PHYSICAL_START;
	leval16= htole16(sm->start);
	ec_datagram_write(addr, &leval16, 2);
	
	/* Length */
	addr->addr.position.off = ESC_ADDR_MAP_SYNCMANAGER_NR(nr) |
	    ESC_ADDR_MAP_SYNCMANAGER_LENGTH;
	leval16 = htole16(sm->len);
	ec_datagram_write(addr, &leval16, 2);
	
	addr->addr.position.off = ESC_ADDR_MAP_SYNCMANAGER_NR(nr) |
	    ESC_ADDR_MAP_SYNCMANAGER_CONTROL;
	val8 = sm->ctrl;
	ec_datagram_write(addr, &val8, 1);

	if (sm->len) {
		addr->addr.position.off = ESC_ADDR_MAP_SYNCMANAGER_NR(nr) |
		   ESC_ADDR_MAP_SYNCMANAGER_ACTIVATE;
		val8 = ESC_SYNCMANAGER_ACTIVATE_ENABLE;
		ec_datagram_write(addr, &val8, 1);
		log_send(LOG_T_DEBUG, "%s: sm%d: activated", dev->name, nr);
	}

	return 0;
}

